@@ -339,7 +339,7 @@ sub register {
 
 		} elsif ( $code ) {
 			$self->error( "Failed with code $code" );
-			invoke_callback( $cb_final, 'FAIL', code => $code );
+			invoke_callback( $cb_final, 'FAIL', code => $code, packet => $packet );
 		} elsif ( $errno ) {
 			$self->error( "Failed with error $errno" );
 			invoke_callback( $cb_final, 'FAIL', errno => $errno );
@@ -362,22 +362,24 @@ sub register {
 ###########################################################################
 # create new call
 # and waits until the INVITE is completed (e.g final response received)
-# Args: ($self,$to;%args)
-#   $to: sip address of peer
+# Args: ($self,$ctx;%args)
+#   $ctx: \%ctx context describing the call or sip address of peer
 #   %args: see Net::SIP::Simple::Call::invite
 # Returns: $call
 #   $call: Net::SIP::Simple::Call
 ###########################################################################
 sub invite {
 	my Net::SIP::Simple $self = shift;
-	my ($to,%args) = @_;
+	my ($ctx,%args) = @_;
+	my $to = ref($ctx) ? $ctx->{to} : $ctx;
 	$to || croak( "need peer of call" );
 	if ( $to !~m{\s} && $to !~m{\@} ) {;
 		croak( "no domain and no fully qualified to" ) if ! $self->{domain};
-		$to = "$to <sip:$to\@$self->{domain}>"
+		$to = "$to <sip:$to\@$self->{domain}>";
+		$ctx->{to} = $to if $ctx;
 	}
-	my $call = Net::SIP::Simple::Call->new( $self,$to );
-	$call->reinvite(%args );
+	my $call = Net::SIP::Simple::Call->new( $self,$ctx||$to );
+	$call->reinvite(%args);
 	return $call;
 }
 
@@ -454,7 +456,7 @@ sub listen {
 	$self->{endpoint}->set_application( [ $receive,$self,\%args] );
 
 	# in case listener should provide authorization put Authorizer in between
-	if ( my $auth = _make_auth_from_args($self,%args) ) {
+	if ( my $auth = _make_auth_from_args($self,\%args) ) {
 		$self->create_chain([$auth,$self->{endpoint}]);
 	}
 }
@@ -462,25 +464,36 @@ sub listen {
 
 ###########################################################################
 # create authorization if args say so
-# Args: ($self,%args)
-#   %args:
+# Args: ($self,$args)
+#   %$args:
 #     auth_user2pass: see user2pass in Net::SIP::Authorize 
 #     auth_user2a1:   see user2a1 in Net::SIP::Authorize 
 #     auth_realm:     see realm in Net::SIP::Authorize 
 #     auth_.... :     see Net::SIP::Authorize
-# Returns: authorizer if auth_* args given
-###########################################################################
+# Returns: authorizer if auth_* args given, removes auth_ args from hash
+##########################################################################
 sub _make_auth_from_args {
-	my ($self,%args) = @_;
+	my ($self,$args) = @_;
 
-	my %auth = map { m{^auth_(.+)} ? ($1 => $args{$_}):() } keys %args;
+	my %auth = 
+		map { m{^auth_(.+)} ? ($1 => delete $args->{$_}):() } 
+		keys %$args;
 	my $i_am_proxy = delete $auth{i_am_proxy};
 
-	return if ! %auth;
+	return %auth && $self->create_auth(%auth);
+}
+
+###########################################################################
+# setup authorization for use in chain
+# Args: ($self,%args)
+#   %args:  see Net::SIP::Authorize
+# Returns: authorizer object
+##########################################################################
+sub create_auth {
+	my ($self,%args) = @_;
 	return Net::SIP::Authorize->new(
 		dispatcher => $self->{dispatcher},
-		i_am_proxy => $i_am_proxy,
-		%auth,
+		%args,
 	);
 }
 
@@ -498,12 +511,18 @@ sub _make_auth_from_args {
 ###########################################################################
 sub create_registrar {
 	my Net::SIP::Simple $self = shift;
+	my %args = @_;
+	my $auth = _make_auth_from_args($self,\%args);
+
 	my $registrar = Net::SIP::Registrar->new(
 		dispatcher => $self->{dispatcher},
-		@_
+		%args
 	);
-	if ( my $auth = _make_auth_from_args($self,@_) ) {
-		$self->create_chain([$auth,$registrar])
+	if ( $auth ) {
+		$registrar = $self->create_chain(
+			[$auth,$registrar], 
+			methods => ['REGISTER']
+		)
 	} else {
 		$self->{dispatcher}->set_receiver( $registrar );
 	}
@@ -519,12 +538,17 @@ sub create_registrar {
 ###########################################################################
 sub create_stateless_proxy {
 	my Net::SIP::Simple $self = shift;
+	my %args = @_;
+
+	$args{auth_i_am_proxy} = 1;
+	my $auth = _make_auth_from_args($self,\%args);
+
 	my $proxy = Net::SIP::StatelessProxy->new(
 		dispatcher => $self->{dispatcher},
-		@_
+		%args
 	);
-	if ( my $auth = _make_auth_from_args($self,auth_i_am_proxy => 1, @_) ) {
-		$self->create_chain([$auth,$proxy])
+	if ( $auth ) {
+		$proxy = $self->create_chain([$auth,$proxy])
 	} else {
 		$self->{dispatcher}->set_receiver($proxy);
 	}
